@@ -22,40 +22,24 @@ const (
 // DownloadFunc downloads a specific Bazel binary to the given location and returns the absolute path.
 type DownloadFunc func(destDir, destFile string) (string, error)
 
-// ReleaseFilter filters Bazel versions based on specific criteria.
-type ReleaseFilter func(matchesSoFar int, currentVersion string) bool
+// LTSFilter filters Bazel versions based on specific criteria.
+type LTSFilter func(string) bool
 
-func lastNReleases(max int) ReleaseFilter {
-	return func(matchesSoFar int, currentVersion string) bool {
-		return max < 1 || matchesSoFar < max
-	}
+// FilterOpts represents options relevant to filtering Bazel versions.
+type FilterOpts struct {
+	MaxResults int
+	Track      int
+	Filter     LTSFilter
 }
 
-// filterReleasesByTrack only works reliably if iterating on Bazel versions in descending order.
-func filterReleasesByTrack(track int) ReleaseFilter {
-	prefix := fmt.Sprintf("%d.", track)
-	return func(matchesSoFar int, currentVersion string) bool {
-		return matchesSoFar == 0 && strings.HasPrefix(currentVersion, prefix)
-	}
-}
+// LTSRepo represents a repository that stores LTS Bazel releases and their candidates.
+type LTSRepo interface {
+	// GetLTSVersions returns a list of all available LTS release (candidates) that match the given filter options.
+	// Warning: Filters only work reliably if the versions are processed in descending order!
+	GetLTSVersions(bazeliskHome string, opts *FilterOpts) ([]string, error)
 
-// ReleaseRepo represents a repository that stores LTS Bazel releases.
-type ReleaseRepo interface {
-	// GetReleaseVersions returns a list of all available release versions that match the given filter function.
-	// Warning: the filter only works reliably if the versions are processed in descending order!
-	GetReleaseVersions(bazeliskHome string, filter ReleaseFilter) ([]string, error)
-
-	// DownloadRelease downloads the given Bazel version into the specified location and returns the absolute path.
-	DownloadRelease(version, destDir, destFile string, config config.Config) (string, error)
-}
-
-// CandidateRepo represents a repository that stores Bazel release candidates.
-type CandidateRepo interface {
-	// GetCandidateVersions returns the versions of all available release candidates.
-	GetCandidateVersions(bazeliskHome string) ([]string, error)
-
-	// DownloadCandidate downloads the given Bazel release candidate into the specified location and returns the absolute path.
-	DownloadCandidate(version, destDir, destFile string, config config.Config) (string, error)
+	// DownloadLTS downloads the given Bazel version into the specified location and returns the absolute path.
+	DownloadLTS(version, destDir, destFile string, config config.Config) (string, error)
 }
 
 // ForkRepo represents a repository that stores a fork of Bazel (releases).
@@ -70,10 +54,8 @@ type ForkRepo interface {
 // CommitRepo represents a repository that stores Bazel binaries built at specific commits.
 // It can also return the hashes of the most recent commits that passed Bazel CI pipelines successfully.
 type CommitRepo interface {
-	// GetLastGreenCommit returns the most recent commit at which a Bazel binary passed a specific Bazel CI pipeline.
-	// If downstreamGreen is true, the pipeline is https://buildkite.com/bazel/bazel-at-head-plus-downstream, otherwise
-	// it's https://buildkite.com/bazel/bazel-bazel
-	GetLastGreenCommit(bazeliskHome string, downstreamGreen bool) (string, error)
+	// GetLastGreenCommit returns the most recent commit at which a Bazel binary is successfully built.
+	GetLastGreenCommit(bazeliskHome string) (string, error)
 
 	// DownloadAtCommit downloads a Bazel binary built at the given commit into the specified location and returns the absolute path.
 	DownloadAtCommit(commit, destDir, destFile string, config config.Config) (string, error)
@@ -90,8 +72,7 @@ type RollingRepo interface {
 
 // Repositories offers access to different types of Bazel repositories, mainly for finding and downloading the correct version of Bazel.
 type Repositories struct {
-	Releases        ReleaseRepo
-	Candidates      CandidateRepo
+	LTS             LTSRepo
 	Fork            ForkRepo
 	Commits         CommitRepo
 	Rolling         RollingRepo
@@ -107,22 +88,20 @@ func (r *Repositories) ResolveVersion(bazeliskHome, fork, version string, config
 
 	if vi.IsFork {
 		return r.resolveFork(bazeliskHome, vi, config)
-	} else if vi.IsRelease {
-		return r.resolveRelease(bazeliskHome, vi, config)
-	} else if vi.IsCandidate {
-		return r.resolveCandidate(bazeliskHome, vi, config)
+	} else if vi.IsLTS {
+		return r.resolveLTS(bazeliskHome, vi, config)
 	} else if vi.IsCommit {
 		return r.resolveCommit(bazeliskHome, vi, config)
 	} else if vi.IsRolling {
 		return r.resolveRolling(bazeliskHome, vi, config)
 	}
 
-	return "", nil, fmt.Errorf("Unsupported version identifier '%s'", version)
+	return "", nil, fmt.Errorf("unsupported version identifier '%s'", version)
 }
 
 func (r *Repositories) resolveFork(bazeliskHome string, vi *versions.Info, config config.Config) (string, DownloadFunc, error) {
-	if vi.IsRelative && (vi.IsCandidate || vi.IsCommit) {
-		return "", nil, errors.New("forks do not support last_rc, last_green and last_downstream_green")
+	if vi.IsRelative && (vi.MustBeCandidate || vi.IsCommit) {
+		return "", nil, errors.New("forks do not support last_rc and last_green")
 	}
 	lister := func(bazeliskHome string) ([]string, error) {
 		return r.Fork.GetVersions(bazeliskHome, vi.Fork)
@@ -137,35 +116,39 @@ func (r *Repositories) resolveFork(bazeliskHome string, vi *versions.Info, confi
 	return version, downloader, nil
 }
 
-func (r *Repositories) resolveRelease(bazeliskHome string, vi *versions.Info, config config.Config) (string, DownloadFunc, error) {
+var IsRelease = func(version string) bool {
+	return !strings.Contains(version, "rc")
+}
+
+var IsCandidate = func(version string) bool {
+	return strings.Contains(version, "rc")
+}
+
+func (r *Repositories) resolveLTS(bazeliskHome string, vi *versions.Info, config config.Config) (string, DownloadFunc, error) {
+	opts := &FilterOpts{
+		// Optimization: only fetch last (x+1) releases if the version is "latest-x".
+		MaxResults: vi.LatestOffset + 1,
+		Track:      vi.TrackRestriction,
+	}
+
+	if vi.MustBeRelease {
+		opts.Filter = IsRelease
+	} else if vi.MustBeCandidate {
+		opts.Filter = IsCandidate
+	} else {
+		// Wildcard -> can be either release or candidate
+		opts.Filter = func(v string) bool { return true }
+	}
+
 	lister := func(bazeliskHome string) ([]string, error) {
-		var filter ReleaseFilter
-		if vi.TrackRestriction > 0 {
-			// Optimization: only fetch matching releases if an LTS track is specified.
-			filter = filterReleasesByTrack(vi.TrackRestriction)
-		} else {
-			// Optimization: only fetch last (x+1) releases if the version is "latest-x".
-			filter = lastNReleases(vi.LatestOffset + 1)
-		}
-		return r.Releases.GetReleaseVersions(bazeliskHome, filter)
+		return r.LTS.GetLTSVersions(bazeliskHome, opts)
 	}
 	version, err := resolvePotentiallyRelativeVersion(bazeliskHome, lister, vi)
 	if err != nil {
 		return "", nil, err
 	}
 	downloader := func(destDir, destFile string) (string, error) {
-		return r.Releases.DownloadRelease(version, destDir, destFile, config)
-	}
-	return version, downloader, nil
-}
-
-func (r *Repositories) resolveCandidate(bazeliskHome string, vi *versions.Info, config config.Config) (string, DownloadFunc, error) {
-	version, err := resolvePotentiallyRelativeVersion(bazeliskHome, r.Candidates.GetCandidateVersions, vi)
-	if err != nil {
-		return "", nil, err
-	}
-	downloader := func(destDir, destFile string) (string, error) {
-		return r.Candidates.DownloadCandidate(version, destDir, destFile, config)
+		return r.LTS.DownloadLTS(version, destDir, destFile, config)
 	}
 	return version, downloader, nil
 }
@@ -174,7 +157,7 @@ func (r *Repositories) resolveCommit(bazeliskHome string, vi *versions.Info, con
 	version := vi.Value
 	if vi.IsRelative {
 		var err error
-		version, err = r.Commits.GetLastGreenCommit(bazeliskHome, vi.IsDownstream)
+		version, err = r.Commits.GetLastGreenCommit(bazeliskHome)
 		if err != nil {
 			return "", nil, fmt.Errorf("cannot resolve last green commit: %v", err)
 		}
@@ -213,7 +196,7 @@ func resolvePotentiallyRelativeVersion(bazeliskHome string, lister listVersionsF
 
 	index := len(available) - 1 - vi.LatestOffset
 	if index < 0 {
-		return "", fmt.Errorf("cannot resolve version \"%s\": There are not enough matching Bazel releases (%d)", vi.Value, len(available))
+		return "", fmt.Errorf("cannot resolve version %q: There are not enough matching Bazel releases (%d)", vi.Value, len(available))
 	}
 	sorted := versions.GetInAscendingOrder(available)
 	return sorted[index], nil
@@ -236,6 +219,7 @@ func (r *Repositories) DownloadFromBaseURL(baseURL, version, destDir, destFile s
 	return httputil.DownloadBinary(url, destDir, destFile, config)
 }
 
+// BuildURLFromFormat returns a Bazel download URL based on formatURL.
 func BuildURLFromFormat(config config.Config, formatURL, version string) (string, error) {
 	osName, err := platforms.DetermineOperatingSystem()
 	if err != nil {
@@ -296,19 +280,13 @@ func (r *Repositories) DownloadFromFormatURL(config config.Config, formatURL, ve
 }
 
 // CreateRepositories creates a new Repositories instance with the given repositories. Any nil repository will be replaced by a dummy repository that raises an error whenever a download is attempted.
-func CreateRepositories(releases ReleaseRepo, candidates CandidateRepo, fork ForkRepo, commits CommitRepo, rolling RollingRepo, supportsBaseURL bool) *Repositories {
+func CreateRepositories(lts LTSRepo, fork ForkRepo, commits CommitRepo, rolling RollingRepo, supportsBaseURL bool) *Repositories {
 	repos := &Repositories{supportsBaseURL: supportsBaseURL}
 
-	if releases == nil {
-		repos.Releases = &noReleaseRepo{err: errors.New("Bazel LTS releases are not supported")}
+	if lts == nil {
+		repos.LTS = &noLTSRepo{err: errors.New("Bazel LTS releases & candidates are not supported")}
 	} else {
-		repos.Releases = releases
-	}
-
-	if candidates == nil {
-		repos.Candidates = &noCandidateRepo{err: errors.New("Bazel release candidates are not supported")}
-	} else {
-		repos.Candidates = candidates
+		repos.LTS = lts
 	}
 
 	if fork == nil {
@@ -332,31 +310,19 @@ func CreateRepositories(releases ReleaseRepo, candidates CandidateRepo, fork For
 	return repos
 }
 
-// The whole point of the structs below this line is that users can simply call repos.Releases.GetReleaseVersions()
-// (etc) without having to worry whether `Releases` points at an actual repo.
+// The whole point of the structs below this line is that users can simply call repos.LTS.GetLTSVersions()
+// (etc) without having to worry whether `LTS` points at an actual repo.
 
-type noReleaseRepo struct {
+type noLTSRepo struct {
 	err error
 }
 
-func (nrr *noReleaseRepo) GetReleaseVersions(bazeliskHome string, filter ReleaseFilter) ([]string, error) {
-	return nil, nrr.err
+func (nolts *noLTSRepo) GetLTSVersions(bazeliskHome string, opts *FilterOpts) ([]string, error) {
+	return nil, nolts.err
 }
 
-func (nrr *noReleaseRepo) DownloadRelease(version, destDir, destFile string, config config.Config) (string, error) {
-	return "", nrr.err
-}
-
-type noCandidateRepo struct {
-	err error
-}
-
-func (ncc *noCandidateRepo) GetCandidateVersions(bazeliskHome string) ([]string, error) {
-	return nil, ncc.err
-}
-
-func (ncc *noCandidateRepo) DownloadCandidate(version, destDir, destFile string, config config.Config) (string, error) {
-	return "", ncc.err
+func (nolts *noLTSRepo) DownloadLTS(version, destDir, destFile string, config config.Config) (string, error) {
+	return "", nolts.err
 }
 
 type noForkRepo struct {
@@ -375,7 +341,7 @@ type noCommitRepo struct {
 	err error
 }
 
-func (nlgr *noCommitRepo) GetLastGreenCommit(bazeliskHome string, downstreamGreen bool) (string, error) {
+func (nlgr *noCommitRepo) GetLastGreenCommit(bazeliskHome string) (string, error) {
 	return "", nlgr.err
 }
 
