@@ -17,7 +17,8 @@ const (
 )
 
 var (
-	releasePattern       = regexp.MustCompile(`^(\d+)\.(x|\d+\.\d+)$`)
+	releasePattern       = regexp.MustCompile(`^(\d+)\.\d+\.\d+$`)
+	trackPattern         = regexp.MustCompile(`^(\d+)\.(x|\*)$`)
 	patchPattern         = regexp.MustCompile(`^(\d+\.\d+\.\d+)-([\w\d]+)$`)
 	candidatePattern     = regexp.MustCompile(`^(\d+\.\d+\.\d+)rc(\d+)$`)
 	rollingPattern       = regexp.MustCompile(`^\d+\.0\.0-pre\.\d{8}(\.\d+){1,2}$`)
@@ -27,9 +28,11 @@ var (
 
 // Info represents a structured Bazel version identifier.
 type Info struct {
-	IsRelease, IsCandidate, IsCommit, IsFork, IsRolling, IsRelative, IsDownstream bool
-	Fork, Value                                                                   string
-	LatestOffset, TrackRestriction                                                int
+	MustBeRelease, MustBeCandidate bool
+	IsLTS, IsRolling               bool
+	IsCommit, IsFork, IsRelative   bool
+	Fork, Value                    string
+	LatestOffset, TrackRestriction int
 }
 
 // Parse extracts and returns structured information about the given Bazel version label.
@@ -37,48 +40,50 @@ func Parse(fork, version string) (*Info, error) {
 	vi := &Info{Fork: fork, Value: version, IsFork: isFork(fork)}
 
 	if m := releasePattern.FindStringSubmatch(version); m != nil {
-		vi.IsRelease = true
-		if m[2] == "x" {
-			track, err := strconv.Atoi(m[1])
-			if err != nil {
-				return nil, fmt.Errorf("invalid version %q, expected something like '5.2.1' or '5.x'", version)
-			}
-			vi.IsRelative = true
-			vi.TrackRestriction = track
+		vi.IsLTS = true
+		vi.MustBeRelease = true
+	} else if m := trackPattern.FindStringSubmatch(version); m != nil {
+		track, err := strconv.Atoi(m[1])
+		if err != nil {
+			return nil, fmt.Errorf("invalid version %q, expected something like '5.x' or '5.*'", version)
 		}
+		vi.IsLTS = true
+		vi.MustBeRelease = (m[2] == "x")
+		vi.IsRelative = true
+		vi.TrackRestriction = track
 	} else if patchPattern.MatchString(version) {
-		vi.IsRelease = true
+		vi.IsLTS = true
+		vi.MustBeRelease = true
 	} else if m := latestReleasePattern.FindStringSubmatch(version); m != nil {
-		vi.IsRelease = true
+		vi.IsLTS = true
+		vi.MustBeRelease = true
 		vi.IsRelative = true
 		if m[1] != "" {
 			offset, err := strconv.Atoi(m[1])
 			if err != nil {
-				return nil, fmt.Errorf("invalid version \"%s\", could not parse offset: %v", version, err)
+				return nil, fmt.Errorf("invalid version %q, could not parse offset: %v", version, err)
 			}
 			vi.LatestOffset = offset
 		}
 	} else if candidatePattern.MatchString(version) {
-		vi.IsCandidate = true
+		vi.IsLTS = true
+		vi.MustBeCandidate = true
 	} else if version == "last_rc" {
-		vi.IsCandidate = true
+		vi.IsLTS = true
+		vi.MustBeCandidate = true
 		vi.IsRelative = true
 	} else if commitPattern.MatchString(version) {
 		vi.IsCommit = true
 	} else if version == "last_green" {
 		vi.IsCommit = true
 		vi.IsRelative = true
-	} else if version == "last_downstream_green" {
-		vi.IsCommit = true
-		vi.IsRelative = true
-		vi.IsDownstream = true
 	} else if rollingPattern.MatchString(version) {
 		vi.IsRolling = true
 	} else if version == "rolling" {
 		vi.IsRolling = true
 		vi.IsRelative = true
 	} else {
-		return nil, fmt.Errorf("Invalid version '%s'", version)
+		return nil, fmt.Errorf("invalid version '%s'", version)
 	}
 	return vi, nil
 }
@@ -106,7 +111,11 @@ func GetInAscendingOrder(versions []string) []string {
 	return sorted
 }
 
+func MatchCommitPattern(version string) bool {
+	return commitPattern.MatchString(version)
+}
+
 // IsCommit returns whether the given version refers to a commit.
 func IsCommit(version string) bool {
-	return version == "last_green" || version == "last_downstream_green" || commitPattern.MatchString(version)
-}
\ No newline at end of file
+	return version == "last_green" || commitPattern.MatchString(version)
+}
